
/*
   This Program finds the unique stationary distribution of a linear
   Markov-Chain. That is a Markov-Chain with n states and transition
   probabilities

      p_{0,1}    =  1
      p_{n,n-1}  =  1
      p_{i,j}    =  0 for |i-j| > 1
      p_{i,j}   =/= 0 for |i-j| = 1
      p_{i,i}    may be =/= 0
      p_{i,i+1} + p_{i,i-1} = 1 for all i = 1 .. n-1

   This program works by explicitly solving the system of linear equations
   defined by (P^T-E)x=0. The running time is in O(n), because the matrix
   has only 3 cells =/= 0 for every row.

   A unique solution always exists, because the transposed probability
   matrix has rank n-1 (so the solution-space is 1-dimensional) and the sum
   of the distributions' components must be 1, so the scale for the (one)
   basis-vector of the solution-space can be determined uniquely.
*/

#include<vector>
#include<iostream>
using namespace std; 

int main(int argc, char** argv) {
	int n;
	cerr << "\nn  >  ";
	cin >> n;
	vector<vector<float> > M(n+1, vector<float>(3,0.0));
	vector<float> f(n+1,0.0);
	cout << "\n";

	// ------------------------------------------- read transition probabilities

	M[1][0] = 1; // from the left and right end, there is only 1 direction
	M[n-1][2] = 1;

	for(int i = 1; i < n; i++) {
		cerr << "p_{" << i << "," << i-1 << "}  >  ";
		cin >> M[i-1][2];

		if(M[i-1][2] > 0.0 && M[i-1][2] < 1.0) {
			cerr << "p_{" << i << "," << i+1 << "} > ";
			cin >> M[i+1][0];
			if(M[i+1][0] > 0.0 && M[i+1][0] < 1.0 && M[i-1][2]+M[i+1][0] <= 1.0) {
				M[i][1] = 0.0 - M[i-1][2] - M[i+1][0];
				cerr << "p_{" << i << "," << i << "} :=  " << M[i][1]+1.0;
			} else {
				cerr << argv[0] << " ERROR: the sum of the probabilities must be in (0,1]!\n";
				i--;
			}
		} else {
			cerr << argv[0] << " ERROR: only values in (0,1) are allowed!\n";
			i--;
		}
	}
	cout << "\n";

	// ------------------------- determine factor (f) between pi_{i-1} and p_{i}

	for(int i = n; i > 0; --i) {
		f[i] = - M[i][0] / M[i][1];
		cout << "\\pi_{" << i << "} = " << f[i] << " * \\pi_{" << i-1 << "}\\\\\n";

		M[i-1][1] += M[i-1][2] * f[i];
		M[i-1][2] = 0.0;
	}
	f[0] = 1;
	cout << "\n";

	// -------- determine factor between pi_{0} (instead of pi_{i-1}) and pi_{i}

	float prod = 1;
	cout << "\\pi_{0} = 1 * \\pi_{0}\\\\\n";
	for(int i = 1; i <= n; i++) {
		f[i] *= f[i-1];
		cout << "\\pi_{" << i << "} = " << f[i] << " * \\pi_{0}\\\\\n";
	}

	cout << "\n(" << f[0] << "x";
	for(int i = 1; i <= n; i++)
		cout << ", " << f[i] << "x";
	cout << ") is stationary for all $x\\in\\mathbb{R}$\\\\\n";

	// ---------------- select the x such that x*f[0]+...+x*f[n] = 1 (x = 1/sum)

	float sum = 0;
	for(int i = 0; i <= n; i++) {
		sum += f[i];
	}
	cout << "select x = " << 1/sum
	     << ", because with this x, the sum of the components is 1.\\\\\n\n";

	// ------------------------------------------------------------ print result

	f[0] /= sum;
	cout << "\\pi = (" << f[0];
	for(int i = 1; i <= n; i++) {
		f[i] /= sum;
		cout << ", " << f[i];
	}
	cout << ")\\\\\n\n";

	// -------------------------------------------------------- thats all, folks

	return 0;
}

